@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import (absolute_import, division, print_function)
-
 import math
 from sympy import Interval
 from sympy.calculus.singularities import is_increasing, is_decreasing
@@ -15,7 +12,11 @@ methods).
 """
 
 class SumApprox(Optimization):
-    """ Approximates sum by neglecting small terms
+    """
+    Approximates sum by neglecting small terms.
+
+    Explanation
+    ===========
 
     If terms are expressions which can be determined to be monotonic, then
     bounds for those expressions are added.
@@ -51,7 +52,7 @@ class SumApprox(Optimization):
     """
 
     def __init__(self, bounds, reltol, **kwargs):
-        super(SumApprox, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.bounds = bounds
         self.reltol = reltol
 
@@ -100,7 +101,7 @@ class SumApprox(Optimization):
 
 
 class SeriesApprox(Optimization):
-    """ Approximates functions by expanding them as a series
+    """ Approximates functions by expanding them as a series.
 
     Parameters
     ==========
@@ -138,7 +139,7 @@ class SeriesApprox(Optimization):
 
     """
     def __init__(self, bounds, reltol, max_order=4, n_point_checks=4, **kwargs):
-        super(SeriesApprox, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.bounds = bounds
         self.reltol = reltol
         self.max_order = max_order
