@@ -3,7 +3,7 @@ from sympy.core.add import Add
 from sympy.core.cache import cacheit
 from sympy.core.function import (
     Function, ArgumentIndexError, _coeff_isneg,
-    expand_mul, FunctionClass)
+    expand_mul, FunctionClass, PoleError)
 from sympy.core.logic import fuzzy_and, fuzzy_not, fuzzy_or
 from sympy.core.mul import Mul
 from sympy.core.numbers import Integer, Rational
@@ -32,6 +32,10 @@ class ExpBase(Function):
     unbranched = True
     _singularities = (S.ComplexInfinity,)
 
+    @property
+    def kind(self):
+        return self.exp.kind
+
     def inverse(self, argindex=1):
         """
         Returns the inverse function of ``exp(x)``.
@@ -418,12 +422,12 @@ class exp(ExpBase, metaclass=ExpMeta):
         sympy.functions.elementary.complexes.re
         sympy.functions.elementary.complexes.im
         """
-        import sympy
+        from sympy.functions.elementary.trigonometric import cos, sin
         re, im = self.args[0].as_real_imag()
         if deep:
             re = re.expand(deep, **hints)
             im = im.expand(deep, **hints)
-        cos, sin = sympy.cos(im), sympy.sin(im)
+        cos, sin = cos(im), sin(im)
         return (exp(re)*cos, exp(re)*sin)
 
     def _eval_subs(self, old, new):
@@ -473,19 +477,21 @@ class exp(ExpBase, metaclass=ExpMeta):
     def _eval_nseries(self, x, n, logx, cdir=0):
         # NOTE Please see the comment at the beginning of this file, labelled
         #      IMPORTANT.
-        from sympy import ceiling, limit, oo, Order, powsimp, Wild, expand_complex
+        from sympy import ceiling, limit, Order, powsimp, Wild, expand_complex
         arg = self.exp
         arg_series = arg._eval_nseries(x, n=n, logx=logx)
         if arg_series.is_Order:
             return 1 + arg_series
         arg0 = limit(arg_series.removeO(), x, 0)
-        if arg0 in [-oo, oo]:
+        if arg0 is S.NegativeInfinity:
+            return Order(x**n, x)
+        if arg0 is S.Infinity:
             return self
         t = Dummy("t")
         nterms = n
         try:
-            cf = Order(arg.as_leading_term(x), x).getn()
-        except NotImplementedError:
+            cf = Order(arg.as_leading_term(x, logx=logx), x).getn()
+        except (NotImplementedError, PoleError):
             cf = 0
         if cf and cf > 0:
             nterms = ceiling(n/cf)
@@ -512,26 +518,14 @@ class exp(ExpBase, metaclass=ExpMeta):
             l.append(g.removeO())
         return Add(*l)
 
-    def _eval_as_leading_term(self, x, cdir=0):
-        from sympy import Order
-        arg = self.args[0]
-        if arg.is_Add:
-            return Mul(*[exp(f).as_leading_term(x) for f in arg.args])
-        arg_1 = arg.as_leading_term(x)
-        if Order(x, x).contains(arg_1):
-            return S.One
-        if Order(1, x).contains(arg_1):
-            return exp(arg_1)
-        ####################################################
-        # The correct result here should be 'None'.        #
-        # Indeed arg in not bounded as x tends to 0.       #
-        # Consequently the series expansion does not admit #
-        # the leading term.                                #
-        # For compatibility reasons, the return value here #
-        # is the original function, i.e. exp(arg),         #
-        # instead of None.                                 #
-        ####################################################
-        return exp(arg)
+    def _eval_as_leading_term(self, x, logx=None, cdir=0):
+        arg = self.args[0].cancel().as_leading_term(x, logx=logx)
+        arg0 = arg.subs(x, 0)
+        if arg0 is S.NaN:
+            arg0 = arg.limit(x, 0)
+        if arg0.is_infinite is False:
+            return exp(arg0)
+        raise PoleError("Cannot expand %s around 0" % (self))
 
     def _eval_rewrite_as_sin(self, arg, **kwargs):
         from sympy import sin
@@ -988,7 +982,7 @@ class log(Function):
         try:
             a, b = arg.leadterm(x)
             s = arg.nseries(x, n=n+b, logx=logx)
-        except (ValueError, NotImplementedError):
+        except (ValueError, NotImplementedError, PoleError):
             s = arg.nseries(x, n=n, logx=logx)
             while s.is_Order:
                 n += 1
@@ -1038,17 +1032,23 @@ class log(Function):
             res -= 2*I*S.Pi
         return res + Order(x**n, x)
 
-    def _eval_as_leading_term(self, x, cdir=0):
-        from sympy import I, im
-        arg = self.args[0].together()
-        x0 = arg.subs(x, 0)
+    def _eval_as_leading_term(self, x, logx=None, cdir=0):
+        from sympy import I, im, re
+        arg0 = self.args[0].together()
+
+        arg = arg0.as_leading_term(x, cdir=cdir)
+        x0 = arg0.subs(x, 0)
+        if (x0 is S.NaN and logx is None):
+            x0 = arg.limit(x, 0, dir='-' if re(cdir).is_negative else '+')
+        if x0 in (S.NegativeInfinity, S.Infinity):
+            raise PoleError("Cannot expand %s around 0" % (self))
         if x0 == 1:
-            return (arg - S.One).as_leading_term(x)
+            return (arg0 - S.One).as_leading_term(x)
         if cdir != 0:
-            cdir = self.args[0].dir(x, cdir)
-        if x0.is_real and x0.is_negative and im(cdir) < 0:
-            return self.func(x0) -2*I*S.Pi
-        return self.func(arg.as_leading_term(x))
+            cdir = arg0.dir(x, cdir)
+        if x0.is_real and x0.is_negative and im(cdir).is_negative:
+            return self.func(x0) - 2*I*S.Pi
+        return self.func(arg)
 
 
 class LambertW(Function):
@@ -1173,6 +1173,14 @@ class LambertW(Function):
         else:
             return s.is_algebraic
 
+    def _eval_as_leading_term(self, x, logx=None, cdir=0):
+        if len(self.args) == 1:
+            arg = self.args[0]
+            arg0 = arg.subs(x, 0).cancel()
+            if not arg0.is_zero:
+                return self.func(arg0)
+            return arg.as_leading_term(x)
+
     def _eval_nseries(self, x, n, logx, cdir=0):
         if len(self.args) == 1:
             from sympy import Order, ceiling, expand_multinomial
