@@ -13,14 +13,12 @@ Medium Term Todo:
   format. This should also use the matrix slots.
 """
 
-from __future__ import print_function, division
-
 from itertools import chain
 import random
 
 from sympy import Add, I, Integer, Mul, Pow, sqrt, Tuple
 from sympy.core.numbers import Number
-from sympy.core.compatibility import is_sequence, u, unicode, range
+from sympy.core.compatibility import is_sequence
 from sympy.printing.pretty.stringpict import prettyForm, stringPict
 
 from sympy.physics.quantum.anticommutator import AntiCommutator
@@ -137,8 +135,8 @@ class Gate(UnitaryOperator):
 
     _label_separator = ','
 
-    gate_name = u('G')
-    gate_name_latex = u('G')
+    gate_name = 'G'
+    gate_name_latex = 'G'
 
     #-------------------------------------------------------------------------
     # Initialization/creation
@@ -288,7 +286,7 @@ class Gate(UnitaryOperator):
         return '%s(%s)' % (self.gate_name, label)
 
     def _pretty(self, printer, *args):
-        a = stringPict(unicode(self.gate_name))
+        a = stringPict(self.gate_name)
         b = self._print_label_pretty(printer, *args)
         return self._print_subscript_pretty(a, b)
 
@@ -319,8 +317,8 @@ class CGate(Gate):
 
     """
 
-    gate_name = u('C')
-    gate_name_latex = u('C')
+    gate_name = 'C'
+    gate_name_latex = 'C'
 
     # The values this class controls for.
     control_value = Integer(1)
@@ -423,10 +421,10 @@ class CGate(Gate):
         controls = self._print_sequence_pretty(
             self.controls, ',', printer, *args)
         gate = printer._print(self.gate)
-        gate_name = stringPict(unicode(self.gate_name))
+        gate_name = stringPict(self.gate_name)
         first = self._print_subscript_pretty(gate_name, controls)
         gate = self._print_parens_pretty(gate)
-        final = prettyForm(*first.right((gate)))
+        final = prettyForm(*first.right(gate))
         return final
 
     def _latex(self, printer, *args):
@@ -446,9 +444,9 @@ class CGate(Gate):
         for c in self.controls:
             circ_plot.control_point(gate_idx, int(c))
         if self.simplify_cgate:
-            if self.gate.gate_name == u('X'):
+            if self.gate.gate_name == 'X':
                 self.gate.plot_gate_plus(circ_plot, gate_idx)
-            elif self.gate.gate_name == u('Z'):
+            elif self.gate.gate_name == 'Z':
                 circ_plot.control_point(gate_idx, self.targets[0])
             else:
                 self.gate.plot_gate(circ_plot, gate_idx)
@@ -499,8 +497,8 @@ class UGate(Gate):
         target qubits and U is a unitary matrix with dimension of
         len(targets).
     """
-    gate_name = u('U')
-    gate_name_latex = u('U')
+    gate_name = 'U'
+    gate_name_latex = 'U'
 
     #-------------------------------------------------------------------------
     # Initialization
@@ -558,7 +556,7 @@ class UGate(Gate):
     def _pretty(self, printer, *args):
         targets = self._print_sequence_pretty(
             self.targets, ',', printer, *args)
-        gate_name = stringPict(unicode(self.gate_name))
+        gate_name = stringPict(self.gate_name)
         return self._print_subscript_pretty(gate_name, targets)
 
     def _latex(self, printer, *args):
@@ -618,8 +616,8 @@ class IdentityGate(OneQubitGate):
     ========
 
     """
-    gate_name = u('1')
-    gate_name_latex = u('1')
+    gate_name = '1'
+    gate_name_latex = '1'
 
     def get_target_matrix(self, format='sympy'):
         return matrix_cache.get_matrix('eye2', format)
@@ -654,8 +652,8 @@ class HadamardGate(HermitianOperator, OneQubitGate):
     sqrt(2)*|00>/2 + sqrt(2)*|11>/2
 
     """
-    gate_name = u('H')
-    gate_name_latex = u('H')
+    gate_name = 'H'
+    gate_name_latex = 'H'
 
     def get_target_matrix(self, format='sympy'):
         if _normalized:
@@ -694,8 +692,8 @@ class XGate(HermitianOperator, OneQubitGate):
     ========
 
     """
-    gate_name = u('X')
-    gate_name_latex = u('X')
+    gate_name = 'X'
+    gate_name_latex = 'X'
 
     def get_target_matrix(self, format='sympy'):
         return matrix_cache.get_matrix('X', format)
@@ -733,8 +731,8 @@ class YGate(HermitianOperator, OneQubitGate):
     ========
 
     """
-    gate_name = u('Y')
-    gate_name_latex = u('Y')
+    gate_name = 'Y'
+    gate_name_latex = 'Y'
 
     def get_target_matrix(self, format='sympy'):
         return matrix_cache.get_matrix('Y', format)
@@ -761,8 +759,8 @@ class ZGate(HermitianOperator, OneQubitGate):
     ========
 
     """
-    gate_name = u('Z')
-    gate_name_latex = u('Z')
+    gate_name = 'Z'
+    gate_name_latex = 'Z'
 
     def get_target_matrix(self, format='sympy'):
         return matrix_cache.get_matrix('Z', format)
@@ -789,8 +787,8 @@ class PhaseGate(OneQubitGate):
     ========
 
     """
-    gate_name = u('S')
-    gate_name_latex = u('S')
+    gate_name = 'S'
+    gate_name_latex = 'S'
 
     def get_target_matrix(self, format='sympy'):
         return matrix_cache.get_matrix('S', format)
@@ -817,8 +815,8 @@ class TGate(OneQubitGate):
     ========
 
     """
-    gate_name = u('T')
-    gate_name_latex = u('T')
+    gate_name = 'T'
+    gate_name_latex = 'T'
 
     def get_target_matrix(self, format='sympy'):
         return matrix_cache.get_matrix('T', format)
@@ -867,7 +865,7 @@ class CNotGate(HermitianOperator, CGate, TwoQubitGate):
 
     """
     gate_name = 'CNOT'
-    gate_name_latex = u('CNOT')
+    gate_name_latex = 'CNOT'
     simplify_cgate = True
 
     #-------------------------------------------------------------------------
@@ -973,7 +971,7 @@ class SwapGate(TwoQubitGate):
 
     """
     gate_name = 'SWAP'
-    gate_name_latex = u('SWAP')
+    gate_name_latex = 'SWAP'
 
     def get_target_matrix(self, format='sympy'):
         return matrix_cache.get_matrix('SWAP', format)
@@ -1230,7 +1228,6 @@ def gate_sort(circuit):
                         new_args = (circuit.args[:i] + (circuit.args[i + 1],) +
                                    (circuit.args[i],) + circuit.args[i + 2:])
                         circuit = Mul(*new_args)
-                        circ_array = circuit.args
                         changes = True
                         break
                     if AntiCommutator(first_base, second_base).doit() == 0:
@@ -1238,7 +1235,6 @@ def gate_sort(circuit):
                                    (circuit.args[i],) + circuit.args[i + 2:])
                         sign = Integer(-1)**(first_exp*second_exp)
                         circuit = sign*Mul(*new_args)
-                        circ_array = circuit.args
                         changes = True
                         break
     return circuit
