@@ -11,7 +11,7 @@ Please see the description of the individual functions for further
 details and examples.
 
 References
-~~~~~~~~~~
+==========
 
 .. [Regge58] 'Symmetry Properties of Clebsch-Gordan Coefficients',
   T. Regge, Nuovo Cimento, Volume 10, pp. 544 (1958)
@@ -28,13 +28,14 @@ References
   Comput. Phys. Commun., Volume 25, pp. 81-85 (1982)
 
 Credits and Copyright
-~~~~~~~~~~~~~~~~~~~~~
+=====================
 
 This code was taken from Sage with the permission of all authors:
 
 https://groups.google.com/forum/#!topic/sage-devel/M4NZdu-7O38
 
-AUTHORS:
+Authors
+=======
 
 - Jens Rasch (2009-03-24): initial version for Sage
 
@@ -45,12 +46,9 @@ AUTHORS:
 Copyright (C) 2008 Jens Rasch <jyr2000@gmail.com>
 
 """
-from __future__ import print_function, division
-
 from sympy import (Integer, pi, sqrt, sympify, Dummy, S, Sum, Ynm, zeros,
                    Function, sin, cos, exp, I, factorial, binomial,
                    Add, ImmutableMatrix)
-from sympy.core.compatibility import range
 
 # This list of precomputed factorials is needed to massively
 # accelerate future calculations of the various coefficients
@@ -63,15 +61,20 @@ def _calc_factlist(nn):
     massively accelerate future calculations of the various
     coefficients.
 
-    INPUT:
+    Parameters
+    ==========
 
-    -  ``nn`` -  integer, highest factorial to be computed
+    nn : integer
+        Highest factorial to be computed.
 
-    OUTPUT:
+    Returns
+    =======
 
-    list of integers -- the list of precomputed factorials
+    list of integers :
+        The list of precomputed factorials.
 
-    EXAMPLES:
+    Examples
+    ========
 
     Calculate list of factorials::
 
@@ -89,11 +92,14 @@ def wigner_3j(j_1, j_2, j_3, m_1, m_2, m_3):
     r"""
     Calculate the Wigner 3j symbol `\operatorname{Wigner3j}(j_1,j_2,j_3,m_1,m_2,m_3)`.
 
-    INPUT:
+    Parameters
+    ==========
 
-    -  ``j_1``, ``j_2``, ``j_3``, ``m_1``, ``m_2``, ``m_3`` - integer or half integer
+    j_1, j_2, j_3, m_1, m_2, m_3 :
+        Integer or half integer.
 
-    OUTPUT:
+    Returns
+    =======
 
     Rational number times the square root of a rational number.
 
@@ -118,7 +124,8 @@ def wigner_3j(j_1, j_2, j_3, m_1, m_2, m_3):
         ...
         ValueError: m values must be integer or half integer
 
-    NOTES:
+    Notes
+    =====
 
     The Wigner 3j symbol obeys the following symmetry rules:
 
@@ -153,7 +160,8 @@ def wigner_3j(j_1, j_2, j_3, m_1, m_2, m_3):
     - zero for violating any one of the conditions
       `j_1 \ge |m_1|`,  `j_2 \ge |m_2|`,  `j_3 \ge |m_3|`
 
-    ALGORITHM:
+    Algorithm
+    =========
 
     This function uses the algorithm of [Edmonds74]_ to calculate the
     value of the 3j symbol exactly. Note that the formula contains
@@ -161,7 +169,8 @@ def wigner_3j(j_1, j_2, j_3, m_1, m_2, m_3):
     for finite precision arithmetic and only useful for a computer
     algebra system [Rasch03]_.
 
-    AUTHORS:
+    Authors
+    =======
 
     - Jens Rasch (2009-03-24): initial version
     """
@@ -224,31 +233,36 @@ def wigner_3j(j_1, j_2, j_3, m_1, m_2, m_3):
 
 def clebsch_gordan(j_1, j_2, j_3, m_1, m_2, m_3):
     r"""
-    Calculates the Clebsch-Gordan coefficient
+    Calculates the Clebsch-Gordan coefficient.
     `\left\langle j_1 m_1 \; j_2 m_2 | j_3 m_3 \right\rangle`.
 
     The reference for this function is [Edmonds74]_.
 
-    INPUT:
+    Parameters
+    ==========
 
-    -  ``j_1``, ``j_2``, ``j_3``, ``m_1``, ``m_2``, ``m_3`` - integer or half integer
+    j_1, j_2, j_3, m_1, m_2, m_3 :
+        Integer or half integer.
 
-    OUTPUT:
+    Returns
+    =======
 
     Rational number times the square root of a rational number.
 
-    EXAMPLES::
+    Examples
+    ========
 
-        >>> from sympy import S
-        >>> from sympy.physics.wigner import clebsch_gordan
-        >>> clebsch_gordan(S(3)/2, S(1)/2, 2, S(3)/2, S(1)/2, 2)
-        1
-        >>> clebsch_gordan(S(3)/2, S(1)/2, 1, S(3)/2, -S(1)/2, 1)
-        sqrt(3)/2
-        >>> clebsch_gordan(S(3)/2, S(1)/2, 1, -S(1)/2, S(1)/2, 0)
-        -sqrt(2)/2
+    >>> from sympy import S
+    >>> from sympy.physics.wigner import clebsch_gordan
+    >>> clebsch_gordan(S(3)/2, S(1)/2, 2, S(3)/2, S(1)/2, 2)
+    1
+    >>> clebsch_gordan(S(3)/2, S(1)/2, 1, S(3)/2, -S(1)/2, 1)
+    sqrt(3)/2
+    >>> clebsch_gordan(S(3)/2, S(1)/2, 1, -S(1)/2, S(1)/2, 0)
+    -sqrt(2)/2
 
-    NOTES:
+    Notes
+    =====
 
     The Clebsch-Gordan coefficient will be evaluated via its relation
     to Wigner 3j symbols:
@@ -262,7 +276,8 @@ def clebsch_gordan(j_1, j_2, j_3, m_1, m_2, m_3):
     See also the documentation on Wigner 3j symbols which exhibit much
     higher symmetry relations than the Clebsch-Gordan coefficient.
 
-    AUTHORS:
+    Authors
+    =======
 
     - Jens Rasch (2009-03-24): initial version
     """
@@ -277,21 +292,25 @@ def _big_delta_coeff(aa, bb, cc, prec=None):
     Racah symbols. Also checks that the differences are of integer
     value.
 
-    INPUT:
+    Parameters
+    ==========
 
-    -  ``aa`` - first angular momentum, integer or half integer
+    aa :
+        First angular momentum, integer or half integer.
+    bb :
+        Second angular momentum, integer or half integer.
+    cc :
+        Third angular momentum, integer or half integer.
+    prec :
+        Precision of the ``sqrt()`` calculation.
 
-    -  ``bb`` - second angular momentum, integer or half integer
+    Returns
+    =======
 
-    -  ``cc`` - third angular momentum, integer or half integer
+    double : Value of the Delta coefficient.
 
-    -  ``prec`` - precision of the ``sqrt()`` calculation
-
-    OUTPUT:
-
-    double - Value of the Delta coefficient
-
-    EXAMPLES::
+    Examples
+    ========
 
         sage: from sage.functions.wigner import _big_delta_coeff
         sage: _big_delta_coeff(1,1,1)
@@ -329,14 +348,17 @@ def racah(aa, bb, cc, dd, ee, ff, prec=None):
     r"""
     Calculate the Racah symbol `W(a,b,c,d;e,f)`.
 
-    INPUT:
+    Parameters
+    ==========
 
-    -  ``a``, ..., ``f`` - integer or half integer
+    a, ..., f :
+        Integer or half integer.
+    prec :
+        Precision, default: ``None``. Providing a precision can
+        drastically speed up the calculation.
 
-    -  ``prec`` - precision, default: ``None``. Providing a precision can
-       drastically speed up the calculation.
-
-    OUTPUT:
+    Returns
+    =======
 
     Rational number times the square root of a rational number
     (if ``prec=None``), or real number if a precision is given.
@@ -348,7 +370,8 @@ def racah(aa, bb, cc, dd, ee, ff, prec=None):
     >>> racah(3,3,3,3,3,3)
     -1/14
 
-    NOTES:
+    Notes
+    =====
 
     The Racah symbol is related to the Wigner 6j symbol:
 
@@ -360,7 +383,8 @@ def racah(aa, bb, cc, dd, ee, ff, prec=None):
     Please see the 6j symbol for its much richer symmetries and for
     additional properties.
 
-    ALGORITHM:
+    Algorithm
+    =========
 
     This function uses the algorithm of [Edmonds74]_ to calculate the
     value of the 6j symbol exactly. Note that the formula contains
@@ -368,7 +392,8 @@ def racah(aa, bb, cc, dd, ee, ff, prec=None):
     for finite precision arithmetic and only useful for a computer
     algebra system [Rasch03]_.
 
-    AUTHORS:
+    Authors
+    =======
 
     - Jens Rasch (2009-03-24): initial version
     """
@@ -404,14 +429,17 @@ def wigner_6j(j_1, j_2, j_3, j_4, j_5, j_6, prec=None):
     r"""
     Calculate the Wigner 6j symbol `\operatorname{Wigner6j}(j_1,j_2,j_3,j_4,j_5,j_6)`.
 
-    INPUT:
+    Parameters
+    ==========
 
-    -  ``j_1``, ..., ``j_6`` - integer or half integer
+    j_1, ..., j_6 :
+        Integer or half integer.
+    prec :
+        Precision, default: ``None``. Providing a precision can
+        drastically speed up the calculation.
 
-    -  ``prec`` - precision, default: ``None``. Providing a precision can
-       drastically speed up the calculation.
-
-    OUTPUT:
+    Returns
+    =======
 
     Rational number times the square root of a rational number
     (if ``prec=None``), or real number if a precision is given.
@@ -437,7 +465,8 @@ def wigner_6j(j_1, j_2, j_3, j_4, j_5, j_6, prec=None):
         ...
         ValueError: j values must be integer or half integer and fulfill the triangle relation
 
-    NOTES:
+    Notes
+    =====
 
     The Wigner 6j symbol is related to the Racah symbol but exhibits
     more symmetries as detailed below.
@@ -478,7 +507,8 @@ def wigner_6j(j_1, j_2, j_3, j_4, j_5, j_6, prec=None):
 
     - only non-zero if any triple of `j`'s fulfill a triangle relation
 
-    ALGORITHM:
+    Algorithm
+    =========
 
     This function uses the algorithm of [Edmonds74]_ to calculate the
     value of the 6j symbol exactly. Note that the formula contains
@@ -497,14 +527,17 @@ def wigner_9j(j_1, j_2, j_3, j_4, j_5, j_6, j_7, j_8, j_9, prec=None):
     Calculate the Wigner 9j symbol
     `\operatorname{Wigner9j}(j_1,j_2,j_3,j_4,j_5,j_6,j_7,j_8,j_9)`.
 
-    INPUT:
+    Parameters
+    ==========
 
-    -  ``j_1``, ..., ``j_9`` - integer or half integer
+    j_1, ..., j_9 :
+        Integer or half integer.
+    prec : precision, default
+        ``None``. Providing a precision can
+        drastically speed up the calculation.
 
-    -  ``prec`` - precision, default: ``None``. Providing a precision can
-       drastically speed up the calculation.
-
-    OUTPUT:
+    Returns
+    =======
 
     Rational number times the square root of a rational number
     (if ``prec=None``), or real number if a precision is given.
@@ -531,7 +564,8 @@ def wigner_9j(j_1, j_2, j_3, j_4, j_5, j_6, j_7, j_8, j_9, prec=None):
         ...
         ValueError: j values must be integer or half integer and fulfill the triangle relation
 
-    ALGORITHM:
+    Algorithm
+    =========
 
     This function uses the algorithm of [Edmonds74]_ to calculate the
     value of the 3j symbol exactly. Note that the formula contains
@@ -554,6 +588,9 @@ def gaunt(l_1, l_2, l_3, m_1, m_2, m_3, prec=None):
     r"""
     Calculate the Gaunt coefficient.
 
+    Explanation
+    ===========
+
     The Gaunt coefficient is defined as the integral over three
     spherical harmonics:
 
@@ -568,14 +605,17 @@ def gaunt(l_1, l_2, l_3, m_1, m_2, m_3, prec=None):
          \operatorname{Wigner3j}(l_1,l_2,l_3,m_1,m_2,m_3)
         \end{aligned}
 
-    INPUT:
+    Parameters
+    ==========
 
-    -  ``l_1``, ``l_2``, ``l_3``, ``m_1``, ``m_2``, ``m_3`` - integer
+    l_1, l_2, l_3, m_1, m_2, m_3 :
+        Integer.
+    prec - precision, default: ``None``.
+        Providing a precision can
+        drastically speed up the calculation.
 
-    -  ``prec`` - precision, default: ``None``. Providing a precision can
-       drastically speed up the calculation.
-
-    OUTPUT:
+    Returns
+    =======
 
     Rational number times the square root of a rational number
     (if ``prec=None``), or real number if a precision is given.
@@ -600,7 +640,8 @@ def gaunt(l_1, l_2, l_3, m_1, m_2, m_3, prec=None):
         ...
         ValueError: m values must be integer
 
-    NOTES:
+    Notes
+    =====
 
     The Gaunt coefficient obeys the following symmetry rules:
 
@@ -633,7 +674,8 @@ def gaunt(l_1, l_2, l_3, m_1, m_2, m_3, prec=None):
     - non-zero only for an even sum of the `l_i`, i.e.
       `L = l_1 + l_2 + l_3 = 2n` for `n` in `\mathbb{N}`
 
-    ALGORITHM:
+    Algorithms
+    ==========
 
     This function uses the algorithm of [Liberatodebrito82]_ to
     calculate the value of the Gaunt coefficient exactly. Note that
@@ -641,9 +683,10 @@ def gaunt(l_1, l_2, l_3, m_1, m_2, m_3, prec=None):
     therefore unsuitable for finite precision arithmetic and only
     useful for a computer algebra system [Rasch03]_.
 
-    AUTHORS:
+    Authors
+    =======
 
-    - Jens Rasch (2009-03-24): initial version for Sage
+    Jens Rasch (2009-03-24): initial version for Sage.
     """
     if int(l_1) != l_1 or int(l_2) != l_2 or int(l_3) != l_3:
         raise ValueError("l values must be integer")
@@ -712,6 +755,9 @@ def dot_rot_grad_Ynm(j, p, l, m, theta, phi):
     r"""
     Returns dot product of rotational gradients of spherical harmonics.
 
+    Explanation
+    ===========
+
     This function returns the right hand side of the following expression:
 
     .. math ::
@@ -754,17 +800,18 @@ def dot_rot_grad_Ynm(j, p, l, m, theta, phi):
 
 
 def wigner_d_small(J, beta):
-    u"""Return the small Wigner d matrix for angular momentum J.
+    """Return the small Wigner d matrix for angular momentum J.
 
-    INPUT:
+    Explanation
+    ===========
 
-    -  ``J`` - An integer, half-integer, or sympy symbol for the total angular
+    J : An integer, half-integer, or sympy symbol for the total angular
         momentum of the angular momentum space being rotated.
-
-    -  ``beta`` - A real number representing the Euler angle of rotation about
+    beta : A real number representing the Euler angle of rotation about
         the so-called line of nodes. See [Edmonds74]_.
 
-    OUTPUT:
+    Returns
+    =======
 
     A matrix representing the corresponding Euler angle rotation( in the basis
     of eigenvectors of `J_z`).
@@ -890,18 +937,20 @@ def wigner_d_small(J, beta):
 
 
 def wigner_d(J, alpha, beta, gamma):
-    u"""Return the Wigner D matrix for angular momentum J.
+    """Return the Wigner D matrix for angular momentum J.
 
-    INPUT:
+    Explanation
+    ===========
 
-    -  ``J`` - An integer, half-integer, or sympy symbol for the total angular
+    J :
+        An integer, half-integer, or sympy symbol for the total angular
         momentum of the angular momentum space being rotated.
-
-    -  ``alpha``, ``beta``, ``gamma`` - Real numbers representing the Euler
-        angles of rotation about the so-called vertical, line of nodes, and
+    alpha, beta, gamma - Real numbers representing the Euler.
+        Angles of rotation about the so-called vertical, line of nodes, and
         figure axes. See [Edmonds74]_.
 
-    OUTPUT:
+    Returns
+    =======
 
     A matrix representing the corresponding Euler angle rotation( in the basis
     of eigenvectors of `J_z`).
@@ -922,7 +971,6 @@ def wigner_d(J, alpha, beta, gamma):
 
     >>> from sympy.physics.wigner import wigner_d
     >>> from sympy import Integer, symbols, pprint
-    >>> from sympy.physics.wigner import wigner_d_small
     >>> half = 1/Integer(2)
     >>> alpha, beta, gamma = symbols("alpha, beta, gamma", real=True)
     >>> pprint(wigner_d(half, alpha, beta, gamma), use_unicode=True)
