@@ -17,11 +17,11 @@ non-implemented methods. They may also supply new implementations of the
 convenience methods, for example if there are faster algorithms available.
 """
 
-from __future__ import print_function, division
 
 from copy import copy
+from functools import reduce
 
-from sympy.core.compatibility import iterable, reduce
+from sympy.core.compatibility import iterable
 from sympy.polys.agca.ideals import Ideal
 from sympy.polys.domains.field import Field
 from sympy.polys.orderings import ProductOrder, monomial_key
@@ -40,7 +40,7 @@ from sympy.core.basic import _aresame
 ##########################################################################
 
 
-class Module(object):
+class Module:
     """
     Abstract base class for modules.
 
@@ -88,13 +88,11 @@ class Module(object):
         """Generate a quotient module."""
         raise NotImplementedError
 
-    def __div__(self, e):
+    def __truediv__(self, e):
         if not isinstance(e, Module):
             e = self.submodule(*e)
         return self.quotient_module(e)
 
-    __truediv__ = __div__
-
     def contains(self, elem):
         """Return True if ``elem`` is an element of this module."""
         try:
@@ -158,7 +156,7 @@ class Module(object):
         raise NotImplementedError
 
 
-class ModuleElement(object):
+class ModuleElement:
     """
     Base class for module element wrappers.
 
@@ -234,7 +232,7 @@ class ModuleElement(object):
 
     __rmul__ = __mul__
 
-    def __div__(self, o):
+    def __truediv__(self, o):
         if not isinstance(o, self.module.ring.dtype):
             try:
                 o = self.module.ring.convert(o)
@@ -242,8 +240,6 @@ class ModuleElement(object):
                 return NotImplemented
         return self.__class__(self.module, self.div(self.data, o))
 
-    __truediv__ = __div__
-
     def __eq__(self, om):
         if not isinstance(om, self.__class__) or om.module != self.module:
             try:
