@@ -1,12 +1,12 @@
 """Sparse rational function fields. """
 
-from __future__ import print_function, division
 
 from typing import Any, Dict
+from functools import reduce
 
 from operator import add, mul, lt, le, gt, ge
 
-from sympy.core.compatibility import is_sequence, reduce
+from sympy.core.compatibility import is_sequence
 from sympy.core.expr import Expr
 from sympy.core.mod import Mod
 from sympy.core.numbers import Exp1
@@ -54,9 +54,9 @@ def sfield(exprs, *symbols, **options):
     Parameters
     ==========
 
-    exprs : :class:`Expr` or sequence of :class:`Expr` (sympifiable)
-    symbols : sequence of :class:`Symbol`/:class:`Expr`
-    options : keyword arguments understood by :class:`Options`
+    exprs   : py:class:`~.Expr` or sequence of :py:class:`~.Expr` (sympifiable)
+    symbols : sequence of :py:class:`~.Symbol`/:py:class:`~.Expr`
+    options : keyword arguments understood by :py:class:`~.Options`
 
     Examples
     ========
@@ -152,6 +152,12 @@ class FracField(DefaultPrinting):
     def __hash__(self):
         return self._hash
 
+    def index(self, gen):
+        if isinstance(gen, self.dtype):
+            return self.ring.index(gen.to_poly())
+        else:
+            raise ValueError("expected a %s, got %s instead" % (self.dtype,gen))
+
     def __eq__(self, other):
         return isinstance(other, FracField) and \
             (self.symbols, self.ngens, self.domain, self.order) == \
@@ -190,11 +196,27 @@ class FracField(DefaultPrinting):
         if isinstance(element, FracElement):
             if self == element.field:
                 return element
+
+            if isinstance(self.domain, FractionField) and \
+                self.domain.field == element.field:
+                return self.ground_new(element)
+            elif isinstance(self.domain, PolynomialRing) and \
+                self.domain.ring.to_field() == element.field:
+                return self.ground_new(element)
             else:
                 raise NotImplementedError("conversion")
         elif isinstance(element, PolyElement):
             denom, numer = element.clear_denoms()
-            numer = numer.set_ring(self.ring)
+
+            if isinstance(self.domain, PolynomialRing) and \
+                numer.ring == self.domain.ring:
+                numer = self.ring.ground_new(numer)
+            elif isinstance(self.domain, FractionField) and \
+                numer.ring == self.domain.field.to_ring():
+                numer = self.ring.ground_new(numer)
+            else:
+                numer = numer.set_ring(self.ring)
+
             denom = self.ring.ground_new(denom)
             return self.raw_new(numer, denom)
         elif isinstance(element, tuple) and len(element) == 2:
@@ -319,11 +341,9 @@ class FracElement(DomainElement, DefaultPrinting, CantSympify):
     def __ne__(f, g):
         return not f == g
 
-    def __nonzero__(f):
+    def __bool__(f):
         return bool(f.numer)
 
-    __bool__ = __nonzero__
-
     def sort_key(self):
         return (self.denom.sort_key(), self.numer.sort_key())
 
@@ -537,8 +557,6 @@ class FracElement(DomainElement, DefaultPrinting, CantSympify):
         else:
             return f.new(f.numer*g_denom, f.denom*g_numer)
 
-    __div__ = __truediv__
-
     def __rtruediv__(f, c):
         if not f:
             raise ZeroDivisionError
@@ -554,8 +572,6 @@ class FracElement(DomainElement, DefaultPrinting, CantSympify):
         else:
             return f.new(f.denom*g_numer, f.numer*g_denom)
 
-    __rdiv__ = __rtruediv__
-
     def __pow__(f, n):
         """Raise ``f`` to a non-negative power ``n``. """
         if n >= 0:
