@@ -1,4 +1,4 @@
-from sympy import S, Basic, exp, multigamma, pi
+from sympy import S, Basic, exp, multigamma, pi, prod
 from sympy.core.sympify import sympify, _sympify
 from sympy.matrices import (ImmutableMatrix, Inverse, Trace, Determinant,
                             MatrixSymbol, MatrixBase, Transpose, MatrixSet,
@@ -84,20 +84,22 @@ class SampleMatrixScipy:
                 colcov=matrix2numpy(dist.scale_matrix_2, float), size=size, random_state=rand_state)
         }
 
+        sample_shape = {
+            'WishartDistribution': lambda dist: dist.scale_matrix.shape,
+            'MatrixNormalDistribution' : lambda dist: dist.location_matrix.shape
+        }
+
         dist_list = scipy_rv_map.keys()
 
         if dist.__class__.__name__ not in dist_list:
             return None
 
-        samples = []
         if seed is None or isinstance(seed, int):
             rand_state = numpy.random.default_rng(seed=seed)
         else:
             rand_state = seed
-        for _ in range(size[0]):
-            samp = scipy_rv_map[dist.__class__.__name__](dist, size[1] if len(size) > 1 else 1, rand_state)
-            samples.append(samp)
-        return samples
+        samp = scipy_rv_map[dist.__class__.__name__](dist, prod(size), rand_state)
+        return samp.reshape(size + sample_shape[dist.__class__.__name__](dist))
 
 
 class SampleMatrixNumpy:
@@ -114,21 +116,21 @@ class SampleMatrixNumpy:
         numpy_rv_map = {
         }
 
+        sample_shape = {
+        }
+
         dist_list = numpy_rv_map.keys()
 
         if dist.__class__.__name__ not in dist_list:
             return None
 
-        samples = []
         import numpy
         if seed is None or isinstance(seed, int):
             rand_state = numpy.random.default_rng(seed=seed)
         else:
             rand_state = seed
-        for _ in range(size[0]):
-            samp = numpy_rv_map[dist.__class__.__name__](dist, size[1], rand_state)
-            samples.append(samp)
-        return samples
+        samp = numpy_rv_map[dist.__class__.__name__](dist, prod(size), rand_state)
+        return samp.reshape(size + sample_shape[dist.__class__.__name__](dist))
 
 
 class SampleMatrixPymc:
@@ -152,14 +154,21 @@ class SampleMatrixPymc:
                 nu=int(dist.n), S=matrix2numpy(dist.scale_matrix, float))
         }
 
+        sample_shape = {
+            'WishartDistribution': lambda dist: dist.scale_matrix.shape,
+            'MatrixNormalDistribution' : lambda dist: dist.location_matrix.shape
+        }
+
         dist_list = pymc3_rv_map.keys()
 
         if dist.__class__.__name__ not in dist_list:
             return None
-
+        import logging
+        logging.getLogger("pymc3").setLevel(logging.ERROR)
         with pymc3.Model():
             pymc3_rv_map[dist.__class__.__name__](dist)
-            return pymc3.sample(size, chains=1, progressbar=False)[:]['X']
+            samps = pymc3.sample(draws=prod(size), chains=1, progressbar=False, random_seed=seed, return_inferencedata=False, compute_convergence_checks=False)['X']
+        return samps.reshape(size + sample_shape[dist.__class__.__name__](dist))
 
 _get_sample_class_matrixrv = {
     'scipy': SampleMatrixScipy,
@@ -285,11 +294,11 @@ def MatrixGamma(symbol, alpha, beta, scale_matrix):
     >>> M = MatrixGamma('M', a, b, [[2, 1], [1, 2]])
     >>> X = MatrixSymbol('X', 2, 2)
     >>> density(M)(X).doit()
-    3**(-a)*b**(-2*a)*exp(Trace(Matrix([
+    exp(Trace(Matrix([
     [-2/3,  1/3],
-    [ 1/3, -2/3]])*X)/b)*Determinant(X)**(a - 3/2)/(sqrt(pi)*gamma(a)*gamma(a - 1/2))
+    [ 1/3, -2/3]])*X)/b)*Determinant(X)**(a - 3/2)/(3**a*sqrt(pi)*b**(2*a)*gamma(a)*gamma(a - 1/2))
     >>> density(M)([[1, 0], [0, 1]]).doit()
-    3**(-a)*b**(-2*a)*exp(-4/(3*b))/(sqrt(pi)*gamma(a)*gamma(a - 1/2))
+    exp(-4/(3*b))/(3**a*sqrt(pi)*b**(2*a)*gamma(a)*gamma(a - 1/2))
 
 
     References
@@ -369,11 +378,11 @@ def Wishart(symbol, n, scale_matrix):
     >>> W = Wishart('W', n, [[2, 1], [1, 2]])
     >>> X = MatrixSymbol('X', 2, 2)
     >>> density(W)(X).doit()
-    2**(-n)*3**(-n/2)*exp(Trace(Matrix([
+    exp(Trace(Matrix([
     [-1/3,  1/6],
-    [ 1/6, -1/3]])*X))*Determinant(X)**(n/2 - 3/2)/(sqrt(pi)*gamma(n/2)*gamma(n/2 - 1/2))
+    [ 1/6, -1/3]])*X))*Determinant(X)**(n/2 - 3/2)/(2**n*3**(n/2)*sqrt(pi)*gamma(n/2)*gamma(n/2 - 1/2))
     >>> density(W)([[1, 0], [0, 1]]).doit()
-    2**(-n)*3**(-n/2)*exp(-2/3)/(sqrt(pi)*gamma(n/2)*gamma(n/2 - 1/2))
+    exp(-2/3)/(2**n*3**(n/2)*sqrt(pi)*gamma(n/2)*gamma(n/2 - 1/2))
 
     References
     ==========
@@ -568,11 +577,11 @@ def MatrixStudentT(symbol, nu, location_matrix, scale_matrix_1, scale_matrix_2):
     >>> M = MatrixStudentT('M', v, [[1, 2]], [[1, 0], [0, 1]], [1])
     >>> X = MatrixSymbol('X', 1, 2)
     >>> density(M)(X)
-    pi**(-1.0)*gamma(v/2 + 1)*Determinant((Matrix([[-1, -2]]) + X)*(Matrix([
+    gamma(v/2 + 1)*Determinant((Matrix([[-1, -2]]) + X)*(Matrix([
     [-1],
-    [-2]]) + X.T) + Matrix([[1]]))**(-v/2 - 1)*Determinant(Matrix([[1]]))**(-1.0)*Determinant(Matrix([
+    [-2]]) + X.T) + Matrix([[1]]))**(-v/2 - 1)/(pi**1.0*gamma(v/2)*Determinant(Matrix([[1]]))**1.0*Determinant(Matrix([
     [1, 0],
-    [0, 1]]))**(-0.5)/gamma(v/2)
+    [0, 1]]))**0.5)
 
     References
     ==========
